#ifndef _REORDERER_H
#define _REORDERER_H 1

#include <assert.h>
#include <stdint.h>
#include <sys/time.h>

#include <algorithm>
#include <map>
#include <queue>
#include <string>
#include <vector>

#include "protocol.h"

struct GREPacket {
	int seq;
	uint16_t proto;
	std::string data;
	timeval ts;

	bool operator> (const GREPacket &other) const {
		return seq > other.seq;
	}
};

class Reorderer : public Sender {
public:
	Reorderer(Sender* sender);
	void send_packet(uint16_t proto, const std::string& data, int seq);

private:
	void check_ts_discontinuity(uint16_t proto, const std::string &data, bool silence);

	Sender* sender;
	int last_seq;
	timeval last_sent_packet;

	std::priority_queue<GREPacket, std::vector<GREPacket>, std::greater<GREPacket>> packet_buffer;
	std::map<int, int> ccs;
};

#endif  // !defined(_REORDERER_H)
